      program mcgr
!win	use msflib
      character*20 version
      parameter (version='3.1 Windows')
*
***************************************************************************
*
*  Version 3.1 for VMS
*  A Monte Carlo programme for calculating g(r), or partial g(r)'s
*  from experimental F(Q) data.( With gaussian-like distribution of
*                                a move.)
*
*    ( And with 'quadratic' corrections to the experimental data.)
*    ( Extra large r-values possible with this version)
*
*
*     R L McGreevy/M A Howe/L Pusztai
*     Last altered 22-01-1999	by P Zetterstrm
*	Cholesky and 1/Q stuff inserted by A Mellergrd 17-08-1999
*
*  Changes:
*   -- new routine for calculating sin(Qr), this extends the 
*      r-range that can be used.
*   -- New code for proper fitting of scale factors and quadratic
*      (or any polynomial) background by Cholesky decomposition
*      The expression minimized is now
*      beta*Fcon(Q)+alpha(1)+alpha(2)*Q+alpha(3)*Q^2-Fexp(Q)
*      NB: ferr array is set to 1/s.d.^2 to speed this up
*   -- 1/Q term in F(Q) moved into the coeff array, but not for iq=1
*      if Q(1)=0 since then F(Q)=4*pi*rho*integral(r^2*G(r))
*      (coeff1 array introduced to contain Q=0 values)
*
***************************************************************************
*
      parameter (max_tot = 4)
      parameter (max_r = 12000)
      parameter (max_par = 6)
      parameter (max_res = 40)
      parameter (max_zc = 5)
      parameter (max_pc = 5)
      parameter (max_cc = 5)
      parameter (max_ngau = 10)
      parameter (max_ranges = 10)
      parameter (max_dim = 4)
 ! The following two parameters need to have the same values
 ! as in the sqr-function
      parameter (max_q   =1100)
      parameter (max_mr = 15)
      parameter (max_ref   =    5)
      parameter (max_back  =    4)
*
! The following line apply only for VMS-version
      real      minval,maxval
!
!win      real*8    time_start,time_used 
      real      time_start,time_used
      real      q(max_q),peak(max_tot,max_res,max_q)
      real      fexp(max_q,max_tot),fnew(max_q,max_tot)
      real      ferr(max_q,max_tot)
      real      df(max_q,max_tot),fcon(max_q,max_tot)
      real      form(max_q,max_tot),compton(max_q,max_tot)
      real      constant(max_tot) ! ,factor(max_tot)
      real      sigma(max_tot)
      real      gpar(max_r+max_ngau,max_par)
      real      gav(max_r+max_ngau,max_par),gsd(max_r+max_ngau,max_par)
      real      coeff(max_q,max_tot,max_par) ! ,rcut(max_par)
      real      coeff1(max_tot,max_par)
      real      chisqr(max_tot),chisav(max_tot)
      real	  sinqr(0:max_q*max_mr),qinv(max_q)

      real      spar(max_par)
      real      sum_q(max_tot),sum_fexp2(max_tot)
      real      achol(max_ref,max_ref),pchol(max_ref)
      real      dchol(max_ref,max_tot),bchol(max_ref),xchol(max_ref)
      real      alpha(max_back,max_tot),beta(max_tot)
      real      wav(max_tot),ur(max_tot),vr(max_tot),wr(max_tot)
      real      log2
      real      rz1(max_zc),rz2(max_zc)
      real      rp1(max_pc),rp2(max_zc)
      real      rc1(max_cc),rc2(max_cc)
      real      cpar(max_cc),dcpar(max_cc),cconc(max_cc),ccoeff(max_cc)
      real      ccoord(max_cc),csigma(max_cc)
      real	gau(2*max_ngau+1),dggau(2*max_ngau+1)
     -         ,gau_ranges(max_ranges,2*max_ngau+1)
     -         ,gau_sig(max_ranges)
     -         ,r_change(max_ranges)
      real	dggau_mult(2*max_ngau+1)
      integer   izpar(max_zc),ippar(max_pc),icpar(max_cc)
      integer   iq1(max_tot,max_q),iq2(max_tot,max_q)
      integer   iprint !,ir1(max_par)
      integer   nq1(max_tot),nq2(max_tot)
      integer   nq,nr
      integer	  n_gau(max_ranges)
      integer   nbckg(max_tot),nref(max_tot)
      logical   save,conv,exists,accept,resol,contin
      logical   renorm(max_tot),background(max_tot)
      logical   mag(max_tot),comp(max_tot)
      logical   looping,partials,smooth
      logical	  lgau,first
*
**** Declarations for the graphical parts
*
      logical l_graph
      real	rr_gr(max_r),qq_gr(max_q),rmin_gr,rmax_gr,qmin_gr,qmax_gr
      real	gpar_gr(max_r),fexp_gr(max_q),fnew_gr(max_q),back_gr(max_q)
      real	grmin_gr,grmax_gr,fqmin_gr,fqmax_gr ! ,yplot(max(max_q,max_r))
      integer nr_gr,nq_gr,pgopen,idev1,idev2
*
      character*80 file,filesq(max_tot),title
      character*1 c_begin
      parameter (pi=3.1415926)
      parameter (log2=0.69314718)
      logical logret
!win      TYPE (windowconfig) wc
      common /sincom/ sinqr,kq,mrnq,f3
*
*-------------------------------------------------------------------------
*     Initialisation
*
        call rmc_init(iseed,time_start)
!win      logret = SETACTIVEQQ (6)
!win      logret = GETWINDOWCONFIG (wc)
!win	wc%title='Parameters'C
!win	logret = SETWINDOWCONFIG (wc)
!win	if (.not.logret) logret = SETWINDOWCONFIG (wc)
	write(*,*) ' Please Wait... '
*-------------------------------------------------------------------------
*     Obtain name of files to be used
*
! The following line is not used for the Window/PC version
      call rmc_datafile(file)
*
*-------------------------------------------------------------------------
*     Read data for programme
*
      call open_read_ext(4,file,'.dat')
      inquire(4,name=file)
      read (4,'(a)') title
      read (4,*) contin
      read (4,*) l_graph
      if (l_graph) then
         read (4,*) i_graph
      else
	 i_graph=100
      endif
      read (4,*) rho
      read (4,*) partials
      read (4,*) npar
      if (npar.gt.max_par) stop 'Too many partials'
      read (4,*) nzc
      if (nzc.gt.max_zc) stop 'Too many zero constraints'
      do izc=1,nzc
        read (4,*) izpar(izc),rz1(izc),rz2(izc)
      enddo
      read (4,*) delta
      read(4,*) mr,rmax
      if (mr.gt.max_mr) then
	 stop 'Mr to large, should be less than max_mr'
      endif
      read (4,*) save
      if (save) read (4,*) nsave
      read (4,*) conv
      read (4,*) npc
      if (npc.gt.max_pc) stop 'Too many positivity constraints'
      do ipc=1,npc
        read (4,*) ippar(ipc),rp1(ipc),rp2(ipc)
      enddo
      read (4,*) ncc
      if (ncc.gt.max_cc) stop 'Too many coordination constraints'
      do icc=1,ncc
        read (4,*) icpar(icc),rc1(icc),rc2(icc),cconc(icc),
     -             ccoeff(icc),ccoord(icc),csigma(icc)
      enddo
      read (4,*) smooth

! Parts of this IF-case moved after the exp. data is read
      if (smooth) then
        read (4,*) nsmooth
        if ((nsmooth.eq.1).or.(nsmooth.eq.2)) 
     -         read (4,*) weight1,weight2,r0
        if (nsmooth.eq.3) then 
          lgau = .true.
***************************      gauss
          read (4,*) n_ranges
          if (n_ranges.gt.max_ranges+1) stop 
     -                        'Too many r ranges for Gaussian.'
          do i_ranges = 1,n_ranges
            read (4,*) gau_sig(i_ranges),r_change(i_ranges)
!**
!*                 input: FWHM; the code would use 'real' sigma's...
!*
!             gau_sig(i_ranges) = gau_sig(i_ranges)/2.354 
!*
!             n_gau(i_ranges)   = int(3.*gau_sig(i_ranges)/dr)
!             if (n_gau(i_ranges).gt.max_ngau) then
!                write (*,*) 'Too many points for a Gaussian.' 
!                write (*,*) '(Decrase sigma, or increase r spacing.)'
!                stop
!             endif
          enddo
!*
!***        r_change(n_ranges) should be .ge. real(nr)*dr; if not....  ******
!*           
!          if (r_change(n_ranges).lt.real(nr)*dr) then          
!            write (*,*) 
!     -         'Range of Gaussians smaller than specified r-range.'
!            write (*,*) 
!     -         '(nr*dr should equal to (or less than) rch(nranges))'
!            stop 
!          endif
*
        endif
*************************    gauss end
      else
        weight1=0.
        weight2=1.
      endif
      read (4,*) resol
      read (4,*) iprint
      read (4,*) timelim,timesav
      read (4,*) ntot1,ntot2
      ntot=ntot1+ntot2
      if (npar.gt.ntot) stop 'Not enough data sets'
      if (ntot.gt.max_tot) stop 'Too many total F(Q)s'
      do itot=1,ntot1
         read (4,'(a)') filesq(itot)      
         read (4,*) nq1(itot),nq2(itot)
         read (4,*) constant(itot)             
         read (4,*) (coeff1(itot,ipar),ipar=1,npar) 
         read (4,*) sigma(itot)   
         if (resol) read (4,*) wav(itot),ur(itot),vr(itot),wr(itot)
         if (save) read (4,*) chisav(itot)
         read (4,*) renorm(itot)
         read (4,*) background(itot)
	   if (background(itot)) then
           read (4,*) nbckg(itot)
	   endif
         read (4,*) mag(itot)
         comp(itot)=.false.
      enddo
      do itot=ntot1+1,ntot
         if (.not.partials) stop 'Incompatible
     -       coefficients and totals'
         read (4,'(a)') filesq(itot)      
         read (4,*) nq1(itot),nq2(itot)
         read (4,*) constant(itot)             
         read (4,*) sigma(itot)   
         if (resol) read (4,*) wav(itot),ur(itot),vr(itot),wr(itot)
         if (save) read (4,*) chisav(itot)
         read (4,*) renorm(itot)
         read (4,*) background(itot)
	   if (background(itot)) then
           read (4,*) nbckg(itot)
	   endif
         read (4,*) comp(itot)
         mag(itot)=.false.
      enddo
      read(4,'(a)') file
      lfile=len(file)
      do while (file(lfile:lfile).eq.' ')
         lfile=lfile-1
      enddo
*
*
      close(4)
*
*-------------------------------------------------------------------------
*     Read experimental data.
*
      do itot=1,ntot1
         call open_read_ext(3,filesq(itot),'.dat')
         read (3,*) nq
         if (nq.gt.max_q) stop 'Too many Q points'
         if (itot.eq.1) nq0=nq
         if (nq.ne.nq0) stop 'Incompatible Q points in data files'
         if (nq.lt.nq2(itot)) nq2(itot)=nq
         read (3,*)
         if (mag(itot)) then
           do iq=1,nq
             read (3,*) q(iq),fexp(iq,itot),form(iq,itot)
             do ipar=1,npar
               coeff(iq,itot,ipar)=coeff1(itot,ipar)
             enddo
             ferr(iq,itot)=sigma(itot)
           enddo
         elseif (sigma(itot).lt.0.) then
           do iq=1,nq
             read (3,*) q(iq),fexp(iq,itot),ferr(iq,itot)
             do ipar=1,npar
               coeff(iq,itot,ipar)=coeff1(itot,ipar)
             enddo
             form(iq,itot)=0.
           enddo
         else
           do iq=1,nq
             read (3,*) q(iq),fexp(iq,itot)
             do ipar=1,npar
               coeff(iq,itot,ipar)=coeff1(itot,ipar)
             enddo
             ferr(iq,itot)=sigma(itot)
             form(iq,itot)=0.
           enddo
         endif
         do iq=1,nq
           compton(iq,itot)=0.
         enddo
         close(3)
         sigma(itot)=abs(sigma(itot))
      enddo
      do itot=ntot1+1,ntot
         call open_read_ext(3,filesq(itot),'.dat')
         read (3,*) nq
         if (nq.gt.max_q) stop 'Too many Q points'
         if (itot.eq.1) nq0=nq
         if (nq.ne.nq0) stop 'Incompatible Q points in data files'
         if (nq.lt.nq2(itot)) nq2(itot)=nq
         read (3,*)
         if (comp(itot)) then
           do iq=1,nq
             read(3,*) q(iq),fexp(iq,itot),
     -       (coeff(iq,itot,ipar),ipar=1,npar),compton(iq,itot)
           enddo
         else
           do iq=1,nq
             read(3,*) q(iq),fexp(iq,itot),
     -       (coeff(iq,itot,ipar),ipar=1,npar)
             compton(iq,itot)=0.
           enddo
         endif
         do iq=1,nq
           form(iq,itot)=0.
           ferr(iq,itot)=sigma(itot)
         enddo
         close(3)
      enddo
!-------------------------------------------------------------
! dr, nr, dq, Nq and kq - from - Qmin, Qmax, Mr and rmax
      dr=2.*pi/(real(mr)*q(nq))
	nr=int(rmax/dr)+1
	if (nr.gt.max_r) then
	  write(*,*) ' nr=',nr,' larger than array dimension',max_r
	  write(*,*) ' Reduce rmax (or increase size parameter max_r) '
	  stop
	endif
	dq=(q(nq)-q(1))/(nq-1)
	nnq=nint(q(nq)/dq)
	kq=nint(q(1)/dq)-1
	if ( abs(q(1)-(kq+1)*dq)/dq.gt.1.e-2 ) then
	  write(*,*) ' Q-values are not multiples of dq '
	  stop
	endif
!------------------------------------------------------------
!Moved from above to be able to use dr and nr in this version

      if (smooth .and. nsmooth.eq.3) then 
         do i_ranges = 1,n_ranges
**
*                input: FWHM; the code would use 'real' sigma's...
*
            gau_sig(i_ranges) = gau_sig(i_ranges)/2.354 
*
            n_gau(i_ranges)   = int(3.*gau_sig(i_ranges)/dr)
            if (n_gau(i_ranges).gt.max_ngau) then
               write (*,*) 'Too many points for a Gaussian.' 
               write (*,*) '(Decrase sigma, or increase r spacing.)'
               stop
            endif
         enddo
*
***       r_change(n_ranges) should be .ge. real(nr)*dr; if not....  ******
*           
         if (r_change(n_ranges).lt.real(nr)*dr) then          
            write (*,*) 
     -         'Range of Gaussians smaller than specified r-range.'
            write (*,*) 
     -         '(nr*dr should equal to (or less than) rch(nranges))'
	      write(*,*) 
     -         'Try a smaller rmax'
            stop 
         endif
      endif
!------------------------------------------------------------
      dof=0.
      if (smooth) dof=dof+real(nr*npar)
      do itot=1,ntot
        do iq=1,nq
           fexp(iq,itot)=fexp(iq,itot)-constant(itot)
        enddo
        sum_q(itot)=real(nq2(itot)-nq1(itot)+1)
        dof=dof+sum_q(itot)
        do iq=nq1(itot),nq2(itot)
            ferr(iq,itot)=1./ferr(iq,itot)/ferr(iq,itot)
        enddo
*
* Set up for Cholesky-decomposition if necessary
*
        if (background(itot)) then
          nref(itot)=nbckg(itot)
          do ii=1,nbckg(itot)
	      dchol(ii,itot)=0.
            do jj=ii,nbckg(itot)
	        achol(ii,jj)=0.
       	  enddo
	    enddo
          do iq=nq1(itot),nq2(itot)
	      qqi=1.
            do ii=1,nbckg(itot)
	        dchol(ii,itot)=dchol(ii,itot)+
     -          fexp(iq,itot)*qqi*ferr(iq,itot)
	        qqj=1.
	        do jj=1,ii-1
	          qqj=qqj*q(iq)
	        enddo
                do jj=ii,nbckg(itot)
  	          achol(ii,jj)=achol(ii,jj)+qqi*qqj*ferr(iq,itot)
	          qqj=qqj*q(iq)
  	        enddo
	        qqi=qqi*q(iq)
            enddo
	  enddo
*
* Cholesky-decompose fnew/fcon-independent part of parameter matrix
*
          do ii=1,nref(itot)
            call choldc(achol,ii,max_ref,pchol)
	    enddo
	  else
          nref(itot)=0
          nbckg(itot)=0
	  endif
          beta(itot)=1.0
          if (renorm(itot)) then
	    nref(itot)=nref(itot)+1
	  else
	    sum_fexp2(itot)=0.0
	    do iq=nq1(itot),nq2(itot)
	      sum_fexp2(itot)=sum_fexp2(itot)+
     -       fexp(iq,itot)*fexp(iq,itot)*ferr(iq,itot)
	    enddo
	  endif
      enddo
      if (partials) then
        gmin=-1.
        do itot=1,ntot
          do ipar=1,npar
            coeff1(itot,ipar)=coeff(1,itot,ipar)
          enddo
        enddo
      else
        gmin=-coeff1(1,1)
        do iq=1,nq
          coeff(iq,1,1)=1.
        enddo
      endif
      if (q(1).eq.0.0) then
	ii1=2
        qinv(1)=1. ! and coeff(1,itot,ipar) is not divided by Q
      else
        ii1=1
      endif
      do iq=ii1,nq
        qinv(iq)=1./q(iq)
        do itot=1,ntot
          do ipar=1,npar
            coeff(iq,itot,ipar)=coeff(iq,itot,ipar)*qinv(iq)
          enddo
        enddo
      enddo
******************          gauss
      if (lgau) then
        do i_ranges = 1,n_ranges
           do i=1,2*n_gau(i_ranges)+1
              gau_ranges(i_ranges,i) = 
     -          exp(-((i-n_gau(i_ranges)-1)*dr)**2/
     -          2/gau_sig(i_ranges)/gau_sig(i_ranges))
           enddo
        enddo
      endif
******************          gauss down to here
*
*-------------------------------------------------------------------------
*     If the file for the stored g(r) values exists read the values
*     from it otherwise initialise g(r) to 1.
*
      if (contin) then
c	inquire(file=file(1:lfile)//'.g',exist=exists)
c      if (exists) then
         call open_read_ext(3,file,'.g')
		 read (3,*) nr
         read (3,*) nsteps,nacc
         read (3,*) dr,(gpar(1,ipar),ipar=1,npar)
         do ir=2,nr                      
            read (3,*) r,(gpar(ir,ipar),ipar=1,npar)
         enddo
         close(3)
      else
         do ipar=1,npar
           do ir=1,nr
              gpar(ir,ipar)=0.
           enddo
         enddo
         nsteps=0
         nacc=0
      endif
*-------------------------------------------------------------------------
*
*-------------------------------------------------------------------------
*     Set g(r) below cut-offs to zero. If positive g(r) only set all
*     negative values to zero
*
      do ir=1,nr
        r=real(ir)*dr
        do izc=1,nzc
          if(r.ge.rz1(izc).and.r.le.rz2(izc))
     -     gpar(ir,izpar(izc))=gmin
        enddo
        do ipc=1,npc
          if(r.ge.rp1(ipc).and.r.le.rp2(ipc)) 
     -     gpar(ir,ippar(ipc))=max(gmin,gpar(ir,ippar(ipc)))
        enddo 
      enddo
*
*-------------------------------------------------------------------------
*     Calculate the coordination if required
*
      do icc=1,ncc
        cpar(icc)=0.
        do ir=1,nr
          r=real(ir)*dr
          if(r.ge.rc1(icc).and.r.le.rc2(icc)) 
     -     cpar(icc)=cpar(icc)+pi*rho*cconc(icc)*(4.*r*r+dr*dr/3.)
     -     *(gpar(ir,icpar(icc))-gmin)*dr/ccoeff(icc)
        enddo
      enddo     
*
*-------------------------------------------------------------------------
*     Calculate the fluctuation term
*
      fluc=0.
      if (smooth) then
        if (nsmooth.eq.1) then
          do ipar=1,npar
            do ir=1,nr-1
              r=dr*real(ir)
              if (r.ge.r0) fluc=fluc+(sqrt(1.+(gpar(ir+1,ipar)-
     -         gpar(ir,ipar))**2)-1.)*r*exp(r/weight2)
            enddo
          enddo
          fluc=fluc*weight1
        elseif (nsmooth.eq.2) then
          do ipar=1,npar
             do ir=2,nr-1
               r=dr*real(ir)
               if (r.ge.r0) fluc=fluc+(gpar(ir+1,ipar)+gpar(ir-1,ipar)
     -          -2.*gpar(ir,ipar))**2*r*exp(r/weight2)
             enddo
          enddo
          fluc=fluc*weight1
*
*********      nsmooth = 3 currently is the gaussian option
*
**        elseif (nsmooth.eq.3) then
**          do ipar=1,npar
**            do ir=2,nr-1
**              r=dr*real(ir)
**              d1=gpar(ir-1,ipar)-gpar(ir,ipar)
**              d2=gpar(ir,ipar)-gpar(ir+1,ipar)
**              fluc=fluc+(1.-sign(1.,d1)*sign(1.,d2))*
**     -         (d1**2+d2**2)*r*exp(r/weight2)
**            enddo
**          enddo
**          fluc=fluc*weight1
****************    nsmooth = 3 taken out
        endif
      endif
*
*-------------------------------------------------------------------------
*
*     If saving read .gsv file, else initialise
*
      if (save) then
        inquire(file=file(1:lfile)//'.gsv',exist=exists)
        if (exists) then
          call open_read_ext(3,file,'.gsv')  
          read (3,*) nr
          read (3,*) nsaved,ntarget
          do ir=1,nr                      
            read (3,*) r,(gav(ir,ipar),ipar=1,npar),
     -       (gsd(ir,ipar),ipar=1,npar)
          enddo
          close(3)
        else
          do ipar=1,npar
            do ir=1,nr
              gav(ir,ipar)=0.
              gsd(ir,ipar)=0.
            enddo
          enddo
          nsaved=0
          ntarget=0
        endif
      endif
*
*-------------------------------------------------------------------------
*     Calculate the sinqr array and the total F(Q)'s for the 
*     initial set of g values.
*
      mrnq=mr*nnq	!Total number of possible sin-values
      if (mrnq.gt.max_q*max_mr) then
        write(*,*) ' Vector sinqr to small (re-dimension) '
        stop
      endif
      f1=4.*pi*rho*dr*dr
      f2=2.*pi/real(mrnq)
      f3=f1*dr
      do imr=0,mrnq
        sinqr(imr)=f1*sin(f2*real(imr))
      end do
      do itot=1,ntot
        do iq=1,nq
          fnew(iq,itot)=form(iq,itot)+compton(iq,itot)
          tmp2=0.
          do ir=1,nr
            tmp1=gpar(ir,1)*coeff(iq,itot,1)
            do ipar=2,npar
              tmp1=tmp1+gpar(ir,ipar)*coeff(iq,itot,ipar)
            enddo            
            tmp2=tmp2+tmp1*real(ir)*sqr(iq,ir)
          enddo
          fnew(iq,itot)=fnew(iq,itot)+tmp2
        enddo
      enddo
      if (resol) then
*     
*     Find the Q range over which the resolution contributes
*
        do itot=1,ntot
          do iq=nq1(itot),nq2(itot)
            ssint=q(iq)*wav(itot)/4./pi
            scost=sqrt(1.-ssint**2)
            stant=ssint/scost
            fw=sqrt(ur(itot)*stant**2+vr(itot)*stant+wr(itot))
            fw=0.5*(fw*pi/180.)*(4.*pi*scost/wav(itot))
            gauss2=4.*log2/fw**2
            iq1(itot,iq)=1
            do while (q(iq1(itot,iq)).lt.q(iq)-3./sqrt(gauss2*2))
              iq1(itot,iq)=iq1(itot,iq)+1
            enddo
            iq2(itot,iq)=nq
            do while (q(iq2(itot,iq)).gt.q(iq)+3./sqrt(gauss2*2))
              iq2(itot,iq)=iq2(itot,iq)-1
            enddo
            if (iq2(itot,iq)-iq1(itot,iq)+1.gt.max_res) 
     -         stop ' Peaks too wide'
            peak_sum=0.
            do iq0=iq1(itot,iq),iq2(itot,iq)
              i=iq0-iq1(itot,iq)+1
              peak(itot,i,iq)=exp(-gauss2*(q(iq)-q(iq0))**2)
              peak_sum=peak_sum+peak(itot,i,iq)
            enddo
            do iq0=iq1(itot,iq),iq2(itot,iq)
              i=iq0-iq1(itot,iq)+1
              peak(itot,i,iq)=peak(itot,i,iq)/peak_sum
            enddo
	      if (iq2(itot,iq).eq.nq) then			 !tmp
	         do iq0=iq1(itot,iq),iq2(itot,iq)	 !tmp
	            i=iq0-iq1(itot,iq)+1			 !tmp
	            peak(itot,i,iq)=0.               !tmp
	         end do								 !tmp
	      endif									 !tmp
          enddo
        enddo
      endif
*
*-------------------------------------------------------------------------
*     Now write out the programme parameters - into text
*                    window.
*
      write (*,500) version,npar,ntot
      write (*,501) nzc
      do izc=1,nzc
        write (*,504) izpar(izc),rz1(izc),rz2(izc)
      enddo
      write(*,502)
      do itot=1,ntot
         write (*,503) itot,(coeff1(itot,ipar),ipar=1,npar)
      enddo
      write (*,520) rho
      do itot=1,ntot
        write (*,521) itot,nq2(itot)-nq1(itot)+1,q(nq1(itot)),
     -                q(nq2(itot)),sigma(itot)
        if (resol) then
          write (*,528) ur(itot),vr(itot),wr(itot)
        else
          write (*,529)
        endif
        if (renorm(itot)) write (*,530)
        if (background(itot)) write (*,540)
      enddo
      write (*,522) nr,dr,nr*dr,delta
      if (conv) write (*,550)
      write (*,560) npc
      do ipc=1,npc
        write (*,561) ippar(ipc),rp1(ipc),rp2(ipc)
      enddo
      write (*,562) ncc
      do icc=1,ncc
        write (*,563) icpar(icc),ccoord(icc),rc1(icc),rc2(icc),
     -                csigma(icc)
        write (*,564) cpar(icc)
      enddo
      if (save) write (*,575) nsave,(chisav(itot),itot=1,ntot)
      if (smooth.and.(nsmooth.eq.1.or.nsmooth.eq.2)) 
     -                    write (*,523) weight1,weight2,r0
      if (lgau) then
         write (*,*)
         write (*,*) ' Applying Gaussian moves.... '
         write (*,*) 
         do i_ranges = 1,n_ranges
            write (*,*) ' Up to ',r_change(i_ranges),'A, sigma is : '
     -                          ,gau_sig(i_ranges)
            write (*,*) '   (with ',2*n_gau(i_ranges)+1
     -                          ,' points in a Gaussian)'
         enddo
         write (*,*)
         write (*,*)
      endif
      write (*,580)
500   format(/' MCGR version ',a/' ===='//
     - 1x,i2,' basis functions fitting to'/
     - 1x,i2,' total F(Q)''s'//
     - ' Cut offs in basis functions are at:')
501   format(//' There are ',i2,' zero constraints ')
502   format(//' Coefficients of basis functions at minimum Q are:')
503   format('     Total ',i2,2x,100(5x,f7.4))
504   format('Partial ',i2,' is zero between ',f6.2,' and ',f6.2)
520   format(//' Number density is ',f6.4,' atoms per cubic Angstrom'//)
521   format(/' Experimental data set ',i2,' is given at ',i4,
     - ' Q points from ',f5.2,' to ',f5.2/' Standard deviation 
     - is ',f7.5)
522   format(/' Using ',i5,' r points spaced at ',f5.3,'A up to ',
     - f8.3,'A'/' Maximum change in rg(r) is ',f6.4/)
523   format(/' Smoothing weight ',f10.2,', exponent ',f6.2,
     - 'A and cut-off ',f6.2/)
528   format(' Resolution parameters (u,v,w) are ',3(f6.2,2x))
529   format(' Not convoluting with experimental resolution function')
530   format(' Renormalising experimental data')
540   format(' Subtracting background from experimental data')
550   format(' Only allowing improved data sets'/)
560   format(/' There are ',i2,' positivity constraints ')
561   format('Partial ',i2,' is positive between ',f8.2,' and ',f8.2)
562   format(/' There are ',i2,' coordination constraints ')
563   format(' Partial ',i2,' is constrained to have a coordination of '/
     - 1x,f6.2,' between ',f6.2,' and ',f6.2,' A with weighting ',f9.5)
564   format(' Starting coordination is ',f6.2)
575   format(' Saving ',i5,' g(r)s with target chi**2 ',10f5.1/)
580   format(/'  beta         a0        a1        a2     Chi-squared')
*
*-------------------------------------------------------------------------
*-------------------------------------------------------------------------
*            Graphics bits...
*
	if (l_graph) then
*
          do jr=1,nr
	     rr_gr(jr)   = jr*dr
             gpar_gr(jr) = gpar(jr,1)
	  enddo 
	  rmin_gr  = rr_gr(1)
	  rmax_gr  = rr_gr(nr)
	  if (rmax_gr.gt.15.0) rmax_gr = 15.0
	  nr_gr = int(rmax_gr/dr) 
          grmin_gr = minval(nr_gr,gpar_gr)
          grmin_gr = grmin_gr - abs(0.1*grmin_gr)
          grmax_gr = maxval(nr_gr,gpar_gr)
          grmax_gr = grmax_gr + abs(0.1*grmax_gr)
	  grmax_gr=max(grmax_gr,1.0)
          itot=1
          do jq=1,nq
	    qq_gr(jq) = q(jq)
            ftmp=0.
            qqi=1.
            do ii=1,nbckg(itot)
	        ftmp=ftmp+alpha(ii,itot)*qqi
	        qqi=qqi*q(jq)
            enddo
	    back_gr(jq)=ftmp
            fexp_gr(jq)=(fexp(jq,itot)-back_gr(jq))/beta(itot)
            fnew_gr(jq)= fnew(jq,itot)
	    enddo
	    qmin_gr  = qq_gr(1)
	    qmax_gr  = qq_gr(nq)
	    nq_gr = nq
            fqmin_gr = min(minval(nq_gr,fexp_gr),minval(nq_gr,fnew_gr),
     -		minval(nq_gr,back_gr))
            fqmin_gr = fqmin_gr - abs(0.2*fqmin_gr)
            fqmax_gr = max(maxval(nq_gr,fexp_gr),maxval(nq_gr,fnew_gr),
     - 		maxval(nq_gr,back_gr))
            fqmax_gr = fqmax_gr + abs(0.2*fqmax_gr)
*
          c_begin='Y'
	  write(*,'(t2,a,$)') 'Begin (Y/N)?'
	  read(*,'(a)') c_begin
	  if (c_begin.eq.'N') stop
          idev1=pgopen('/xwin')
	  if(idev1.le.0) stop 'Graphics error'
          idev2=pgopen('/xwin')
	  if(idev2.le.0) stop 'Graphics error'
	  call pgslct(idev1)
!win        logret = GETWINDOWCONFIG (wc)
!win	    wc%title='G(r)'C
!win	    logret = SETWINDOWCONFIG (wc)
!win	    if (.not.logret) logret = SETWINDOWCONFIG (wc)
          call pgask(.false.)
	  call pgpap(6.0,0.618)
 	  call pgpage
	  call pgenv(rmin_gr,rmax_gr,grmin_gr,grmax_gr,0,0)
	  call pglab('r/A','G(r)',' ')
	  call pgsci(4)
	  call pgline(nr_gr,rr_gr,gpar_gr)
	  call pgsci(1)
	  call pgslct(idev2)
!win        logret = GETWINDOWCONFIG (wc)
!win	    wc%title='S(Q)'C
!win	    logret = SETWINDOWCONFIG (wc)
!win	    if (.not.logret) logret = SETWINDOWCONFIG (wc)
          call pgask(.false.)
	  call pgpap(6.0,0.618)
	  call pgpage
	  call pgenv(qmin_gr,qmax_gr,fqmin_gr,fqmax_gr,0,0)
	  call pglab('Q/A-1','F(Q)',' ')
	  call pgsci(4)
	  call pgline(nq_gr,qq_gr,fnew_gr)
	  call pgsci(3)
	  call pgline(nq_gr,qq_gr,fexp_gr)
	  call pgsci(7)
	  call pgline(nq_gr,qq_gr,back_gr)
	  call pgsci(1)
	endif
*
*---------------------------------------------------------------
*     This is where the loop begins. 
*
!???      call rmc_init(iseed,time_start)
      save_time=min(timelim,timesav)
      chisq0=1e30
      looping=.true.
      first=.true.
      do while (looping)
*
*-------------------------------------------------------------------------
*        Change one of the basis g(r)'s 
*
111      jpar=int(ran(iseed)*npar)+1
         jr=int(ran(iseed)*nr)+1
         r=real(jr)*dr
         do izc=1,nzc
           if (jpar.eq.izpar(izc)) then
             if ((r.ge.rz1(izc)).and.(r.le.rz2(izc))) goto 111
           endif
         enddo
         dg=2.*(ran(iseed)-0.5)*delta/r	  !Original g(r)*r change
!         dg=2.*(ran(iseed)-0.5)*delta	  !g(r) change
!         dg=2.*(ran(iseed)-0.5)*delta/(r*r) !g(r)*r**2 change
*
*************                gauss
         if (lgau) then
*
******       which sigma and ngau ?? (depends on the actual jr ...)
*
           j_range = 1
           if (n_ranges.gt.1) then
             do i_ranges = 1,n_ranges
               if (r.le.r_change(i_ranges)) then
                 j_range = i_ranges
                 goto 299
               endif
             enddo
           endif
 299       continue
           ngau = n_gau(j_range)
           do i=1,2*ngau+1
             gau(i) = gau_ranges(j_range,i)
           enddo
*
           iegau = jr+ngau
           isgau = jr-ngau
           do igau = 1,2*ngau+1
             dggau_mult(igau) = 1.0
           enddo
*
**********  If the tail of the gaussian extends the 'r' range ...
*
           do igau = 1,ngau
             if ((jr+igau).gt.nr) dggau_mult(ngau+1+igau) = 0.0
           enddo
           do izc=1,nzc
             if (jpar.eq.izpar(izc)) then
*
**********
*                    If any part of the gaussian falls into the
*                    region of 'r' where g(r) has to be zero, 
*                    then the dggau(.) values concerned should be set to
*                    zero.
**********
* 
               do igau=1,2*ngau+1
                 r_cur = real(isgau+igau-1)*dr
                 if ((r_cur.gt.rz1(izc)).and.(r_cur.lt.rz2(izc))) 
     -              dggau_mult(igau) = 0.0     
               enddo
             endif
           enddo
           do igau=1,2*ngau+1
             dggau(igau) = dg*gau(igau)*dggau_mult(igau)
             if (gpar(isgau+igau-1,jpar)+dggau(igau).lt.gmin) then       
               r = (isgau+igau-1)*dr
               do ipc=1,npc
                 if (jpar.eq.ippar(ipc)) then
                   if ((r.ge.rp1(ipc)).and.(r.le.rp2(ipc))) 
     -                dggau(igau) = gmin-gpar(isgau+igau-1,jpar)
                 endif
               enddo
             endif
           enddo
         else
***********************         gauss down to here
           if (gpar(jr,jpar)+dg.lt.gmin) then       
             do ipc=1,npc
               if (jpar.eq.ippar(ipc)) then
                 if ((r.ge.rp1(ipc)).and.(r.le.rp2(ipc))) 
     -             dg = gmin-gpar(jr,jpar)
               endif
             enddo
           endif
         endif
*-------------------------------------------------------------------------
*        Calculate the change in coordination
*
      cchisq=0.
      do icc=1,ncc
        dcpar(icc)=0.
        if (jpar.eq.icpar(icc)) then
**********************          gauss bits
          if (lgau) then
            do igau=1,2*ngau+1
              r=real(igau+isgau-1)*dr
              if (r.ge.rc1(icc).and.r.le.rc2(icc))
     -          dcpar(icc)=dcpar(icc)+pi*rho*cconc(icc)*
     -          (4.*r*r+dr*dr/3.)*dggau(igau)*dr/ccoeff(icc)
            enddo
**********************          gauss bits down to here
          else
            r=real(jr)*dr
            if (r.ge.rc1(icc).and.r.le.rc2(icc))
     -        dcpar(icc)=pi*rho*cconc(icc)*
     -        (4.*r*r+dr*dr/3.)*dg*dr/ccoeff(icc)
            endif
          endif
          cpar(icc)=cpar(icc)+dcpar(icc)
          cchisq=cchisq+((cpar(icc)-ccoord(icc))/csigma(icc))**2
        enddo

*-------------------------------------------------------------------------
*        Calculate the change in the fluctuation term
*
        if (smooth) then
          if (nsmooth.eq.1) then
            jr1=max(1,jr-1)
            jr2=min(jr,nr-1)
            fluc_old=0.
            do ir=jr1,jr2
              r=dr*real(ir)
              if (r.ge.r0) fluc_old=fluc_old+
     -          (sqrt(1.+(gpar(ir+1,jpar)-
     -          gpar(ir,jpar))**2)-1.)*r*exp(r/weight2)
            enddo
            gpar(jr,jpar)=gpar(jr,jpar)+dg
            fluc_new=0.
            do ir=jr1,jr2
              r=dr*real(ir)
              if (r.ge.r0) fluc_new=fluc_new+
     -          (sqrt(1.+(gpar(ir+1,jpar)-
     -          gpar(ir,jpar))**2)-1.)*r*exp(r/weight2)
            enddo
            dfluc=weight1*(fluc_new-fluc_old)             
          elseif (nsmooth.eq.2) then
            jr1=max(2,jr-1)
            jr2=min(nr-1,jr+1)
            fluc_old=0.
            do ir=jr1,jr2
              r=dr*real(ir)
              if (r.ge.r0) fluc_old=fluc_old+
     -          (gpar(ir+1,jpar)+gpar(ir-1,jpar)
     -          -2.*gpar(ir,jpar))**2*r*exp(r/weight2)
            enddo
            gpar(jr,jpar)=gpar(jr,jpar)+dg
            fluc_new=0.
            do ir=jr1,jr2
              r=dr*real(ir)
              if (r.ge.r0) fluc_new=fluc_new+
     -          (gpar(ir+1,jpar)+gpar(ir-1,jpar)
     -          -2.*gpar(ir,jpar))**2*r*exp(r/weight2)
            enddo
            dfluc=weight1*(fluc_new-fluc_old)
*
**********        nsmooth = 3 currently is the gaussian option
*
**           elseif (nsmooth.eq.3) then
**             jr1=max(2,jr-1)
**             jr2=min(nr-1,jr+1)
**             fluc_old=0.
**             do ir=jr1,jr2
**                r=dr*real(ir)
**               d1=gpar(ir-1,jpar)-gpar(ir,jpar)
**               d2=gpar(ir,jpar)-gpar(ir+1,jpar)
**               fluc_old=fluc_old+(1.-sign(1.,d1)
**     -          *sign(1.,d2))*(d1**2+d2**2)*r*exp(r/weight2)
**             enddo
**             gpar(jr,jpar)=gpar(jr,jpar)+dg
**             fluc_new=0.
**             do ir=jr1,jr2
**                r=dr*real(ir)
**               d1=gpar(ir-1,jpar)-gpar(ir,jpar)
**               d2=gpar(ir,jpar)-gpar(ir+1,jpar)
**               fluc_new=fluc_new+(1.-sign(1.,d1)
**     -          *sign(1.,d2))*(d1**2+d2**2)*r*exp(r/weight2)
**             enddo
**             dfluc=weight1*(fluc_new-fluc_old)
******************     nsmooth = 3 taken out
          endif
        else
          dfluc=0.
          if (.not.lgau) then
            gpar(jr,jpar)=gpar(jr,jpar)+dg
          endif
        endif
        fluc=fluc+dfluc
*
*-------------------------------------------------------------------------
*        Calculate the change in the F(Q)'s
*
*************             gauss bits
        if (lgau) then
          do itot=1,ntot
            do iq=1,nq
              df(iq,itot) = 0.
            enddo
          enddo
          do igau=1,2*ngau+1
            if (dggau(igau).ne.0.0) then
              tmp1=dggau(igau)*real(isgau+igau-1)
              do iq=1,nq
                tmp2=sqr(iq,isgau+igau-1)*tmp1
                do itot=1,ntot
                  df(iq,itot)=df(iq,itot)+tmp2
                enddo
              enddo
            endif
          enddo
          do iq=1,nq
            do itot=1,ntot
              df(iq,itot)=df(iq,itot)*coeff(iq,itot,jpar)
              fnew(iq,itot)=fnew(iq,itot)+df(iq,itot)
            enddo
          enddo
*************             gauss bits down to here
        else
          tmp1=dg*real(jr)
          do iq=1,nq
            tmp2=sqr(iq,jr)*tmp1
            do itot=1,ntot
              df(iq,itot)=coeff(iq,itot,jpar)*tmp2
              fnew(iq,itot)=fnew(iq,itot)+df(iq,itot)
            enddo
          enddo
        endif 
        if (resol) then
          do itot=1,ntot
            do iq=1,nq
              fcon(iq,itot)=0.
            enddo
            do iq=1,nq
              do iq0=iq1(itot,iq),iq2(itot,iq)
                i=iq0-iq1(itot,iq)+1
                fcon(iq0,itot)=fcon(iq0,itot)+fnew(iq,itot)*
     -            peak(itot,i,iq)
              enddo
            enddo
          enddo
        else
          do itot=1,ntot
            do iq=1,nq
              fcon(iq,itot)=fnew(iq,itot)
            enddo
          enddo
        endif
*
*-------------------------------------------------------------------------
*        If required calculate the new renormalisation and offset values
*
        chisq=0.
        do itot=1,ntot
	    if (renorm(itot)) then
	      achol(nref(itot),nref(itot))=0.
	      bchol(nref(itot))=0.
            do ii=1,nbckg(itot)
	        achol(ii,nref(itot))=0.
              bchol(ii)=dchol(ii,itot)
            enddo
            do iq=nq1(itot),nq2(itot)
	        qqi=fcon(iq,itot)*ferr(iq,itot)
	        achol(nref(itot),nref(itot))=
     -          achol(nref(itot),nref(itot))+
     -          fcon(iq,itot)*qqi
	        bchol(nref(itot))=bchol(nref(itot))+
     -          fexp(iq,itot)*qqi
              do ii=1,nbckg(itot)
	          achol(ii,nref(itot))=achol(ii,nref(itot))+qqi
	          qqi=qqi*q(iq)
 	        enddo
	      enddo
            call choldc(achol,nref(itot),max_ref,pchol)
            call cholsl(achol,nref(itot),max_ref,pchol,bchol,xchol)
	      beta(itot)=xchol(nref(itot))
	    elseif (background(itot)) then
            do ii=1,nbckg(itot)
	        bchol(ii)=dchol(ii,itot)
	      enddo
            do iq=nq1(itot),nq2(itot)
      	      qqi=beta(itot)*fcon(iq,itot)*ferr(iq,itot)
              do ii=1,nbckg(itot)
	          bchol(ii)=bchol(ii)-qqi
	          qqi=qqi*q(iq)
	        enddo
            enddo
            call cholsl(achol,nref(itot),max_ref,pchol,bchol,xchol)
          endif
	    do ii=1,nbckg(itot)
	      alpha(ii,itot)=xchol(ii)
	    enddo
*       
*-------------------------------------------------------------------------
*        Calculate chi-squared
*
c this should work, but no.../AM
c	     if (.not.renorm(itot)) then
c	       chisqr(itot)=0.0
c	       do iq=nq1(itot),nq2(itot)
c	         chitmp=beta(itot)*fcon(iq,itot)-fexp(iq,itot)
c	         chisqr(itot)=chisqr(itot)+chitmp*chitmp*ferr(iq,itot)
c	       enddo
c	     else
c	       chisqr(itot)=sum_fexp2(itot)
c	     endif
c	     do ii=1,nref(itot)
c	       chisqr(itot)=chisqr(itot)-xchol(ii)*bchol(ii)
c 	     enddo
          chisqr(itot)=0.
          do iq=nq1(itot),nq2(itot)
            ftmp=beta(itot)*fcon(iq,itot)-fexp(iq,itot)
            qqi=1.
            do ii=1,nbckg(itot)
	        ftmp=ftmp+alpha(ii,itot)*qqi
	        qqi=qqi*q(iq)
	      enddo
            chisqr(itot)=chisqr(itot)+ftmp*ftmp*ferr(iq,itot)
          enddo
**           chisqr(itot)=chisqr(itot)/sigma(itot)/sigma(itot)
          chisq=chisq+chisqr(itot)
        enddo
*-------------------------------------------------------------------------
*        Add the fluctuation and coordination terms
*
        chisq=chisq+fluc+cchisq
*
*-------------------------------------------------------------------------
*        Decide whether to accept new g(r)
*
        accept=.true.
        prat=(chisq0-chisq)*0.5
        if ((conv.and.(prat.lt.0)).or.prat.lt.-20) then
           accept=.false.
        elseif (prat.lt.0) then
          prat=exp(prat)
          if (prat.lt.ran(iseed)) accept=.false.
        endif
*
        nsteps=nsteps+1
        if (.not.accept) then
          if (.not.lgau) then
            gpar(jr,jpar)=gpar(jr,jpar)-dg
          endif
          fluc=fluc-dfluc
          do itot=1,ntot
            do iq=1,nq
              fnew(iq,itot)=fnew(iq,itot)-df(iq,itot)
            enddo
          enddo
          do icc=1,ncc
            cpar(icc)=cpar(icc)-dcpar(icc)
          enddo
        else
*************                gauss
          if (lgau) then
            do igau=1,2*ngau+1
              gpar(isgau+igau-1,jpar) = 
     -                 gpar(isgau+igau-1,jpar)+dggau(igau)
            enddo
          endif
*************
          chisq0=chisq
          nacc=nacc+1
*
*---------------------------------------------------------------
*
*           Graphics bits ....
*
      if (mod(nacc,i_graph).eq.0) then
        if (l_graph) then
          do jr=1,nr
             gpar_gr(jr) = gpar(jr,1)
     	    enddo 
          grmin_gr = minval(nr_gr,gpar_gr)
          grmin_gr = grmin_gr - abs(0.1*grmin_gr)
          grmax_gr = maxval(nr_gr,gpar_gr)
          grmax_gr = grmax_gr + abs(0.1*grmax_gr)
          grmax_gr=max(grmax_gr,1.0)
          itot=1
          do jq=1,nq
            ftmp=0.
            qqi=1.
            do ii=1,nbckg(itot)
	        ftmp=ftmp+alpha(ii,itot)*qqi
	        qqi=qqi*q(jq)
            enddo
	      back_gr(jq)=ftmp
            fexp_gr(jq)=(fexp(jq,itot)-back_gr(jq))/beta(itot)
            fnew_gr(jq)= fnew(jq,itot)
          enddo
          fqmin_gr = min(minval(nq_gr,fexp_gr),minval(nq_gr,fnew_gr),
     -		minval(nq_gr,back_gr))
          fqmin_gr = fqmin_gr - abs(0.2*fqmin_gr)
          fqmax_gr = max(maxval(nq_gr,fexp_gr),maxval(nq_gr,fnew_gr),
     - 		maxval(nq_gr,back_gr))
          fqmax_gr = fqmax_gr + abs(0.2*fqmax_gr)
*
          call pgslct(idev1)
	    call pgpage
	    call pgenv(rmin_gr,rmax_gr,grmin_gr,grmax_gr,0,0)
	    call pglab('r/A','G(r)',' ')
	    call pgsci(4)
	    call pgline(nr_gr,rr_gr,gpar_gr)
	    call pgsci(1)
	    call pgslct(idev2)
	    call pgpage 
	    call pgenv(qmin_gr,qmax_gr,fqmin_gr,fqmax_gr,0,0)
	    call pglab('Q/A-1','F(Q)',' ')
	    call pgsci(4)
	    call pgline(nq_gr,qq_gr,fnew_gr)
	    call pgsci(3)
	    call pgline(nq_gr,qq_gr,fexp_gr)
	    call pgsci(7)
	    call pgline(nq_gr,qq_gr,back_gr)
	    call pgsci(1)
        endif
	endif
cam          if ((nacc.eq.1.or.mod(nacc,iprint).eq.0)) then
          if ((first.or.mod(nacc,iprint).eq.0)) then
            first=.false.
            write (*,*) ' Accepted: ',nacc,'     Generated:',nsteps
            do itot=1,ntot-1
              write(*,600) beta(itot),
     -          (alpha(ii,itot),ii=1,nbckg(itot)),
     -          chisqr(itot)/sum_q(itot)
            enddo
            write(*,590) beta(ntot),
     -        (alpha(ii,ntot),ii=1,nbckg(ntot)),
     -        chisqr(ntot)/sum_q(ntot),
     -        fluc/real(nr*npar),chisq/dof
            do icc=1,ncc
              write (*,*) icc,cpar(icc),ccoord(icc),cchisq
            enddo
*
*---------------------------------------------------------------
*
590         format(7(1x,g10.4))
600         format(5(1x,g10.4))
*
          endif
*
*          If saving g(r) then decide whether to save now
*
          if (save.and.ntarget.eq.0) then
            ntarget=5*nacc
            do itot=1,ntot
              if (chisqr(itot)/sum_q(itot).gt.chisav(itot)) ntarget=0
            enddo
            if (ntarget.ne.0) then
              nsteps=0
              nacc=0
            endif
          endif
          if ((ntarget.ne.0).and.(nacc.ge.ntarget)) then
            ntarget=0
            nsteps=0
            nacc=0
            do ipar=1,npar
              do ir=1,nr
                gav(ir,ipar)=gav(ir,ipar)+gpar(ir,ipar)
                gsd(ir,ipar)=gsd(ir,ipar)+gpar(ir,ipar)**2
              enddo
            enddo
            nsaved=nsaved+1
            write (*,610) nsaved
            if (nsaved.ge.nsave) then
              looping=.false.
              goto 620
            endif
            write (*,615)
            do ipar=1,npar
              do ir=1,nr
                gpar(ir,ipar)=0.
              enddo
            enddo
            nsteps=0
            nacc=0
            do ir=1,nr
              r=real(ir)*dr
              do izc=1,nzc
                if(r.ge.rz1(izc).and.r.le.rz2(izc))
     -            gpar(ir,izpar(izc))=gmin
              enddo
              do ipc=1,npc
                if(r.ge.rp1(ipc).and.r.le.rp2(ipc)) 
     -            gpar(ir,ippar(ipc))=max(gmin,gpar(ir,ippar(ipc)))
              enddo 
            enddo
            fluc=0.
            if (smooth) then
              if (nsmooth.eq.1) then
                do ipar=1,npar
                  do ir=1,nr-1
                    r=dr*real(ir)
                    fluc=fluc+(sqrt(1.+(gpar(ir+1,ipar)-
     -                gpar(ir,ipar))**2)-1.)*r*exp(r/weight2)
                  enddo
                enddo
                fluc=fluc*weight1
              elseif (nsmooth.eq.2) then
                do ipar=1,npar
                  do ir=2,nr-1
                    r=dr*real(ir)
                    fluc=fluc+(gpar(ir+1,ipar)+gpar(ir-1,ipar)
     -               -2.*gpar(ir,ipar))**2*r*exp(r/weight2)
                  enddo
                enddo
                fluc=fluc*weight1
*
**********       nsmooth = 3 currently is the gaussian option
*
**               elseif (nsmooth.eq.3) then
**                 do ipar=1,npar
**                   do ir=1,nr
**                     r=dr*real(ir)
**                     fluc=fluc+abs(gpar(ir,ipar))*r*exp(r/weight2)
**                   enddo
**                 enddo
**
**                 fluc=fluc*weight1
**************          nsmooth = 3 taken out
              endif
            endif
            do itot=1,ntot
              do iq=1,nq
                fnew(iq,itot)=form(iq,itot)+compton(iq,itot)
                tmp2=0.
                do ir=1,nr
                  tmp1=gpar(ir,1)*coeff(iq,itot,1)
                  do ipar=2,npar
                    tmp1=tmp1+gpar(ir,ipar)*coeff(iq,itot,ipar)
                  enddo            
                  tmp2=tmp2+tmp1*real(ir)*sqr(iq,ir)
                enddo
                fnew(iq,itot)=fnew(iq,itot)+tmp2
              enddo
            enddo
            chisq0=1.e30
          endif
*
        endif
610     format(' Saved ',i5, ' sets of values')
615     format(' Resetting G(r) to start')
*
*-------------------------------------------------------------------------
*        check whether it is time to save the current position and 
*        correlation functions; if it is not loop to another move.
*
620     call rmc_time(time_start,time_used)
        if ((time_used.lt.0.or.time_used.ge.save_time)
     -      .or.(save.and.(nsaved.eq.nsave))) then
          write (*,*) 'Saving results ... Do not interrupt'
*
*----------------------------------------------------------------------
*           Store current g(r) values
*
          call open_write(3,file,'.g')
          write (3,*) nr
          write (3,*) nsteps,nacc
          do ir=1,nr
            write (3,*) real(ir)*dr,(gpar(ir,ipar),ipar=1,npar)
          enddo
          close(3)
*
*----------------------------------------------------------------------
*
*           If saving write .gsv file
*
          if (save) then
            call open_write(3,file,'.gsv')
            write (3,*) nr,npar
            write (3,*) nsaved,ntarget
            do ir=1,nr
              write (3,*) real(ir)*dr,(gav(ir,ipar),ipar=1,npar),
     -           (gsd(ir,ipar),ipar=1,npar)
            enddo
            close(3)
          endif
*
*----------------------------------------------------------------------
*           Finally write out results to a file that can be plotted.
*
          call open_write(3,file,'.out')
          write (3,*) 'Results from MCGR'
          write (3,*) 'TITLE'
          write (3,*) title
          write (3,*) ntot,' experimental data sets'
          do itot=1,ntot
            write (3,*) ' Q, S(Q) (MCGR), S(Q) (Expt), Background'
            write (3,*) 'CURVES'
            if (resol) then
              do iq=1,nq
                fcon(iq,itot)=0.
              enddo
              do iq=1,nq
                do iq0=iq1(itot,iq),iq2(itot,iq)
                  i=iq0-iq1(itot,iq)+1
                  fcon(iq0,itot)=fcon(iq0,itot)+fnew(iq,itot)*
     -              peak(itot,i,iq)
                enddo
              enddo
              write (3,*) nq2(itot)-nq1(itot)+1,4
              do iq=nq1(itot),nq2(itot)
                fbck=0.
                qqi=1.
                do ii=1,nbckg(itot)
                  fbck=fbck+alpha(ii,itot)*qqi
                  qqi=qqi*q(iq)
                enddo
                ftmp=(fexp(iq,itot)-fbck)/beta(itot)
                fbck=fbck/beta(itot)
                write (3,*) q(iq),fnew(iq,itot),fcon(iq,itot),
     -              ftmp,fbck
              enddo
            else
              do iq=1,nq
                fcon(iq,itot)=fnew(iq,itot)
              enddo
              write (3,*) nq2(itot)-nq1(itot)+1,3
              do iq=nq1(itot),nq2(itot)
                fbck=0.
                qqi=1.
                do ii=1,nbckg(itot)
                  fbck=fbck+alpha(ii,itot)*qqi
                  qqi=qqi*q(iq)
                enddo
                ftmp=(fexp(iq,itot)-fbck)/beta(itot)
                fbck=fbck/beta(itot)
                write (3,*) q(iq),fcon(iq,itot),ftmp,fbck
              enddo
            endif
          enddo
          write (3,*) 'ENDGROUP'
          write (3,*) npar,' Basis functions'
          write (3,*) ' r, Basis g(r)'
          write (3,*) 'PLOTS'
          write (3,*) nr,npar
          if (partials) then
            do ir=1,nr
              write (3,*) real(ir)*dr,(gpar(ir,ipar)+1.,ipar=1,npar)
            enddo
          else
            do ir=1,nr
              write (3,*) real(ir)*dr,(gpar(ir,ipar),ipar=1,npar)
            enddo
          endif
          write (3,*) 'ENDGROUP'
          if (partials) then
            write (3,*) 'Partial S(Q)''s'
            write (3,*) ' Q, S(Q)'
            write (3,*) 'PLOTS'
            write (3,*) nq,npar
            do iq=1,nq
              do ipar=1,npar
                spar(ipar)=0.
                do ir=1,nr
                  spar(ipar)=spar(ipar)+gpar(ir,ipar)
     -                           *sqr(iq,ir)*real(ir)
                enddo
              enddo
              write (3,*) q(iq),(spar(ipar)*qinv(iq),ipar=1,npar)
            enddo
          endif
          close(3)
          save_time=save_time+timesav
          write (*,*) 'Saving completed'
        endif
*
*----------------------------------------------------------------------
*        Decide whether to continue programme
*
        if ((time_used.lt.0.).or.(time_used.ge.timelim))
     -    looping=.false.
      enddo
*
      if(save.and.(nsaved.eq.nsave)) then
        call rmc_rerun(.false.)
      else
        call rmc_rerun(.true.)
      endif
      stop
      end

*************
*   
*            Matrix inversion
*
*************
      subroutine invert(a,b,idim,isize,itot,max_tot)
      real a(isize,isize),b(isize,isize,max_tot)
      real u(10,10),l(10,10)
      real ui(10,10),li(10,10)
*
*--------- Inverts a matrix by triangular decomposition
*
      do i=1,idim
        do j=1,idim
          u(i,j)=0.
          l(i,j)=0.
          ui(i,j)=0.
          li(i,j)=0.
        enddo
        l(i,i)=1.
        li(i,i)=1.
      enddo
*
*--------- Decompose into upper and lower triangular matrices
*
      do j=1,idim
        do i=1,j
          u(i,j)=a(i,j)
          do k=1,i-1
            u(i,j)=u(i,j)-l(i,k)*u(k,j)
          enddo
        enddo
        do i=j+1,idim
          l(i,j)=a(i,j)/u(j,j)
          do k=1,j-1
            l(i,j)=l(i,j)-l(i,k)*u(k,j)/u(j,j)
          enddo
        enddo
      enddo
*
*--------- Invert triangular matrices
*
      do j=1,idim
        do i=j+1,idim
          do k=j,i-1
            li(i,j)=li(i,j)-l(i,k)*li(k,j)
          enddo
        enddo
        ui(j,j)=1./u(j,j)
        do i=j-1,1,-1
          do k=i+1,j
            ui(i,j)=ui(i,j)-u(i,k)*ui(k,j)/u(i,i)
          enddo
        enddo
      enddo
*
*--------- Multiply to give inverse of original matrix
*
      do i=1,idim
        do j=1,idim
          b(i,j,itot)=ui(i,1)*li(1,j)
          do k=2,idim
            b(i,j,itot)=b(i,j,itot)+ui(i,k)*li(k,j)
          enddo
        enddo
      enddo
*
      return
      end


      function sqr(iq,ir)
 ! The following two parameters need to have the same values
 ! as in the main program
      parameter (max_q = 1100)
      parameter (max_mr = 15)
      real sinqr(0:max_q*max_mr),f3
      integer iq,ir,jq
      common /sincom/ sinqr,kq,mrnq,f3
      jq=iq+kq
      if (jq.ne.0) then
        k=mod(jq*ir,mrnq)
        sqr=sinqr(k)
      else
        sqr=f3*real(ir)
      endif
      return
      end

c******************************************************************************

      SUBROUTINE choldc(a,n,np,p)
! Cholesky decomposition of a symmetric matrix
! Slightly modified NUMREC procedure:
! Calculates only the nth step in the rTr decomposition
      INTEGER n,np
      REAL a(np,np),p(n)
      INTEGER i,j,k
      REAL sum
      do i=1,n
!        do j=i,n
        do j=n,n
          sum=a(i,j)
          do k=i-1,1,-1
            sum=sum-a(i,k)*a(j,k)
          enddo
          if(i.eq.j)then
            if(sum.le.0.)pause 'choldc failed'
            p(i)=sqrt(sum)
          else
            a(j,i)=sum/p(i)
          endif
        enddo
      enddo
      return
      END

c******************************************************************************

      SUBROUTINE cholsl(a,n,np,p,b,x)
! Solves the linear equation system of a Cholesky decomposed
! matrix by forward and backward substitution
      INTEGER n,np
      REAL a(np,np),b(n),p(n),x(n)
      INTEGER i,k
      REAL sum
      do i=1,n
        sum=b(i)
        do k=i-1,1,-1
          sum=sum-a(i,k)*x(k)
        enddo
        x(i)=sum/p(i)
      enddo
      do i=n,1,-1
        sum=x(i)
        do k=i+1,n
          sum=sum-a(k,i)*x(k)
        enddo
        x(i)=sum/p(i)
      enddo
      return
      END

      real function minval(n,x)
! Find minimum value of array x
!
      real x(n)
      fmin=x(1)
      do i=2,n
         fmin=min(fmin,x(i))
      end do
      minval=fmin
      return
      end

      real function maxval(n,x)
! Find maximum value of array x
!
      real x(n)
      fmax=x(1)
      do i=2,n
         fmax=max(fmax,x(i))
      end do
      maxval=fmax
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine open_read_ext opens a file for reading, filling in the
*     supplied extension if none is supplied with the file name.
*
      subroutine open_read_ext(unit,file,extension)
      integer unit
      character*(*) file,extension,temp*80
      temp=file
      i=index(file,'.')
      l=len(file)
      do while (file(l:l).eq.' ')
         l=l-1
      enddo
      if (i.eq.0) temp=file(1:l)//extension
      open (unit,file=temp,status='old')
      return
      end

c******************************************************************************


