! To create the library RMCLIB.OLB include routines from
! RMC2LIB.FOR and RMC_VAX (or RMC_(system)).

*      Contains the following machine dependent routines: 
*        rmc_init        initialises clock and random number generator
*        rmc_datafile    returns name of data file to be read
*        rmc_time        calculates CPU time used
*        rmc_rerun       passes RERUN flag to operating system
*        open_read       opens a file for reading
*        open_write      opens a file for writing 
*        real_to_hex     converts real*4 to IEEE hex string
*        hex_to_real     converts IEEE hex string to real*4
*        open_read_ext   opens a file for reading
*
*-----------------------------------------------------------------------
*
*     Subroutine rmc_data must return the name of the data file to
*     be read by the programme. If possible this will be supplied
*     as a parameter on the command line that invokes the programme
*
      subroutine rmc_datafile(file)
      character*(*) file
      call lib$get_foreign(file)
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine rmc_init must set the seed for the random number 
*     generator and obtain the current cpu clock reading in minutes
*
      subroutine rmc_init(iseed,time_start)
      include '($jpidef)'
      iseed=int(secnds(0.))*2+1
      call lib$getjpi(jpi$_cputim,,,iused)
      time_start=real(iused)/6000.
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine rmc_time, given the initial cpu clock reading, must 
*     return the cpu time used (in minutes). If possible should return
*     a negative value if time limit is about to be reached.
*
*     VAX version is able to check CPU time left and if this is
*     less than 30 seconds returns -time_used
*
      subroutine rmc_time(time_start,time_used)
      include '($jpidef)'
      call lib$getjpi(jpi$_cputim,,,iused)
      time_used=real(iused)/6000.-time_start
      call lib$getjpi(jpi$_cpulim,,,limit)
      left=(limit-iused)/100
      if (limit.gt.0.and.left.lt.30) time_used=-time_used 
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine rmc_rerun, can be used to return the rerun flag to
*     the operating system, if desired.
*
      subroutine rmc_rerun(rerun)
      logical rerun
      character true*4, false*5
      data true,false/'TRUE','FALSE'/
      if (rerun) then
         call lib$set_symbol('RERUN',true)
      else
         call lib$set_symbol('RERUN',false)
      endif
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine open_read opens a file for reading, filling in the
*     supplied extension if none is supplied with the file name.
*     Allows use of system specific facilities of open statement.
*
      subroutine open_read(unit,file,extension)
      integer unit
      character*(*) file,extension
      open (unit,file=file,status='old',defaultfile=extension,
     - readonly)
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine open_write opens a file for writing, filling in the
*     supplied extension if none is supplied with the file name.
*     Allows use of system specific facilities of open statement.
*
      subroutine open_write(unit,file,extension)
      integer unit
      character*(*) file,extension
      open (unit,file=file,status='new',defaultfile=extension,
     - carriagecontrol='list')
      return
      end
*
*-----------------------------------------------------------------------
*
*     Function real_to_hex converts a 32 bit real value to an
*     8 character hex string describing its internal representation.
*     If intermediate files are to be portable this must be
*     converted to IEEE form.
*
*     For VAX F_floating conversion to IEEE entails swapping top two
*     bytes with bottom two bytes and subtracting 2 from the exponent
*
      character*8 function real_to_hex(r)
      character*8 temp
      write (temp,'(z8)') r/4.
      do i=1,8
         if (temp(i:i).eq.' ') temp(i:i)='0'
      enddo
      real_to_hex(1:4)=temp(5:8)
      real_to_hex(5:8)=temp(1:4)
      i=1
      do while (real_to_hex(i:i).eq.'0'.and.i.le.7)
         real_to_hex(i:i)=' '
         i=i+1
      enddo
      return
      end
*
*-----------------------------------------------------------------------
*     Function hex_to_real converts an 8 character hex string in IEEE 
*     format to a 32 bit real value.
*
      real function hex_to_real(hex,error)
      character hex*(*),temp*8
      logical error
      error=.false.
      temp(1:4)=hex(5:8)
      temp(5:8)=hex(1:4)
      read (temp,'(z8)',end=10,err=10) hex_to_real
      hex_to_real=hex_to_real*4.
      return
10    error=.true.
      return
      end
*-----------------------------------------------------------------------
*
*     Subroutine open_read_ext opens a file for reading, filling in the
*     supplied extension if none is supplied with the file name.
*
      subroutine open_read_ext(unit,file,extension)
      integer unit
      character*(*) file,extension,temp*80
      temp=file
      i=index(file,'.')
      l=len(file)
      do while (file(l:l).eq.' ')
         l=l-1
      enddo
      if (i.eq.0) temp=file(1:l)//extension
      open (unit,file=temp,status='old')
      return
      end
*
